#include "dispatch_parallel.h"

#include <assert.h>

#if EFFECTS_USE_GCD
#include <dispatch/dispatch.h>
#endif

#if EFFECTS_USE_CPP11_THREADS
#include <thread>
static void thread_func(
    dispatch_work_func work_func,
    size_t num_iters,
    void* context,
    size_t iter,
    size_t delta_iter)
{
    for (; iter < num_iters; iter += delta_iter)
        work_func(context, iter);
}
#endif

void dispatch_parallel(dispatch_work_func work_func, int num_iters, void* context) {
    assert(num_iters > 0);
#if EFFECTS_USE_GCD
    dispatch_apply_f((size_t)num_iters, dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), context, work_func);
#elif EFFECTS_USE_CPP11_THREADS
    unsigned num_threads = std::thread::hardware_concurrency();
    if (num_threads == 0)
        num_threads = 1;
    
//    std::thread threads[num_threads];
//    for (int i = 0; i < num_threads; ++i)
//        threads[i] = std::thread(thread_func, work_func, (size_t)num_iters, context, (size_t)i, (size_t)num_threads);
//    for (int i = 0; i < num_threads; ++i)
//        threads[i].join();
#else
#pragma omp parallel for
    for (int iter = 0; iter < num_iters; ++iter)
        work_func(context, iter);
#endif
}
